use File::Basename;
use POSIX;

$cmd_type = $ARGV[0];

$ENV{PATH} = "/bin:/usr/bin";  # set path (security thing)

setuid(0); # euid should already be 0, but launchctl wants uid to be 0

# detect version of mac os
$sw_vers = `/usr/bin/sw_vers`;
if ($sw_vers =~ /ProductVersion:\t*(\d*\.\d*)/) {
    $MAC_OS_VERSION = $1;
}
else {
    # if for some reason the regex breaks, default to leopard
    $MAC_OS_VERSION = "10.5";
}

# set screen sharing launchd path
if ($MAC_OS_VERSION eq "10.6") {
    $SS_LAUNCHD_PATH = '/private/etc/ScreenSharing.launchd';
}
else {
    $SS_LAUNCHD_PATH = '/Library/Preferences/com.apple.ScreenSharing.launchd';
}

# set remote management launchd path
if ($MAC_OS_VERSION eq "10.6") {
    $RM_LAUNCHD_PATH = '/private/etc/RemoteManagement.launchd';
}
else {
    $RM_LAUNCHD_PATH = '/Library/Preferences/com.apple.RemoteManagement.launchd';
}

$pre_lion = $MAC_OS_VERSION eq "10.4" || $MAC_OS_VERSION eq "10.5" || $MAC_OS_VERSION eq "10.6";

$VNCSETTINGS_PATH = '/Library/Preferences/com.apple.VNCSettings.txt';
$VNCLEGACYSETTINGS_PATH = '/Library/Preferences/com.apple.RemoteManagement.plist';

if ($cmd_type eq "status")
{
    $status_cmd_type = $ARGV[1];  # ss=screen sharing, rm=remote management 

    if (!$pre_lion)
    {
        $ENV{DYLD_INSERT_LIBRARIES} = dirname($0) . "/libgeteuid_override.dylib";
        $status = (`/bin/launchctl list | grep -E 'com.apple.screensharing\$' >/dev/null 2>&1 && echo "enabled"` eq "enabled\n");
    }
    else
    {
        # screen sharing/remote management are enabled if their launchd file exists
        $ss_running = 0;
        if (-e $SS_LAUNCHD_PATH) { $ss_running = 1; }

        $rm_running = 0;
        if (-e $RM_LAUNCHD_PATH) { $rm_running = 1; }

        $status = 0;
        if ($status_cmd_type eq "ss") { $status = $ss_running; } 
        elsif ($status_cmd_type eq "rm") { $status = $rm_running; }
        else { $status = $ss_running || $rm_running; }
	
		unless (-e $VNCLEGACYSETTINGS_PATH)
		{ 
			# on tiger, if this file does not exist then screen sharing is not running
			if ($MAC_OS_VERSION eq "10.4") { $status = 0; }
		}
    }
    if ($status) { print "enabled\n"; }
    else { print "disabled\n"; }
}
elsif ($cmd_type eq "start") 
{
    if (!$pre_lion)
    {
        `/bin/launchctl load -w /System/Library/LaunchDaemons/com.apple.screensharing.plist >/dev/null 2>&1`;
    }
    else
    {
        # start screen sharing by creating the launchd file with text enabled
        open (LAUNCHD_FILE, ">$SS_LAUNCHD_PATH");
        print LAUNCHD_FILE "enabled\n";
        close(LAUNCHD_FILE);
    }
    exec("dscl . -change /Groups/com.apple.access_screensharing RecordName com.apple.access_screensharing com.apple.access_screensharing-disabled >/dev/null 2>&1");
    exec("dscacheutil -flushcache");
}
elsif ($cmd_type eq "tigerstart")
{
    exec("/System/Library/CoreServices/RemoteManagement/ARDAgent.app/Contents/Resources/kickstart -activate -configure -clientopts -setvnclegacy -vnclegacy yes -restart -agent");
}
elsif ($cmd_type eq "stop") 
{
    if (!$pre_lion)
    {
        `/bin/launchctl unload /System/Library/LaunchDaemons/com.apple.screensharing.plist >/dev/null 2>&1`;
    }
    else
    {
        unlink($SS_LAUNCHD_PATH);  # delete launchd file to stop screen sharing 
    }
    exec("dscl . -change /Groups/com.apple.access_screensharing-disabled RecordName com.apple.access_screensharing-disabled com.apple.access_screensharing >/dev/null 2>&1");
    exec("dscacheutil -flushcache")
}
elsif ($cmd_type eq "tigerstop")
{
    exec("/System/Library/CoreServices/RemoteManagement/ARDAgent.app/Contents/Resources/kickstart -stop -agent");
}
elsif ($cmd_type eq "setpwd")
{
    $pwd = $ARGV[1];
    BEGIN { 
	@k = unpack "C*", pack "H*", "1734516E8BA8C5E2FF1C39567390ADCA"
    }; 
#$_ = <>; 
    $_ = $pwd;
    chomp; 
    s/^(.{8}).*/$1/; 
    @p = unpack "C*", $_; 
    
    open (PWDFILE, ">$VNCSETTINGS_PATH");
    foreach (@k) { 
#    printf "%02X", $_ ^ (shift @p || 0); 
	printf PWDFILE "%02X", $_ ^ (shift @p || 0); 
    }; 
#print "\n";
    print PWDFILE "\n";
    close (PWDFILE);
}
elsif ($cmd_type eq "tigersetpwd")
{
    $pwd = $ARGV[1];
    BEGIN { 
	@k = unpack "C*", pack "H*", "1734516E8BA8C5E2FF1C39567390ADCA"
    }; 
#$_ = <>; 
    $_ = $pwd;
    chomp; 
    s/^(.{8}).*/$1/; 
    @p = unpack "C*", $_; 
    
    $codedpwd = "";
    foreach (@k) { 
#    printf "${codedpwd}%02X", $_ ^ (shift @p || 0); 
	$codedpwd .= sprintf "%02X", $_ ^ (shift @p || 0); 
    }; 
#print "$codedpwd\n";
    $finalcmd = "/System/Library/CoreServices/RemoteManagement/ARDAgent.app/Contents/Resources/kickstart -activate -configure -clientopts -setvncpw -vncpw ".$codedpwd." -setvnclegacy -vnclegacy yes -restart -agent";
    $finalcmd =~ /(.*)/;
    $finalcmd = $1;
    exec($finalcmd);
}
elsif ($cmd_type eq "getpwd" or $cmd_type eq "tigergetpwd")
{
    open (VNCSETTINGS_FILE, $VNCSETTINGS_PATH);
    $line = <VNCSETTINGS_FILE>;
    close (VNCSETTINGS_FILE);

    BEGIN { 
	@k = unpack "C*", pack "H*", "1734516E8BA8C5E2FF1C39567390ADCA"
    }; 

    chomp; 
    $_ = $line;
    @p = unpack "C*", pack "H*", $_; 
    foreach (@k) { 
	printf "%c", $_ ^ (shift @p || 0) 
    }; 
#    print "\n";
}
else 
{
    print "Usage: {status|start|stop|setpwd|getpwd|tigerstart|tigerstop|tigersetpwd|tigergetpwd}\n"
}
